@@ -14,6 +14,7 @@
 """Public API for Xarray-Beam."""
 
 # pylint: disable=g-multiple-import
+from xarray_beam import Mean
 from xarray_beam._src.combiners import (
     MeanCombineFn,
 )
@@ -38,9 +39,10 @@ from xarray_beam._src.rechunk import (
     in_memory_rechunk,
 )
 from xarray_beam._src.zarr import (
+    open_zarr,
+    make_template,
     ChunksToZarr,
     DatasetToZarr,
 )
-from xarray_beam import Mean
 
-__version__ = '0.4.0'
+__version__ = '0.4.1'
