@@ -12,39 +12,96 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 """IO with Zarr via Xarray."""
+import collections
+import dataclasses
 import logging
-from typing import List, Optional, Mapping, Union, MutableMapping
+from typing import Any, List, Optional, Mapping, Tuple, Union, MutableMapping
 
 import apache_beam as beam
-import dataclasses
 import xarray
 
 from xarray_beam._src import core
 from xarray_beam._src import rechunk
 from xarray_beam._src import threadmap
 
-# pylint: disable=logging-format-interpolation
+# pylint: disable=logging-fstring-interpolation
+
+ReadableStore = Union[str, Mapping[str, bytes]]
+WritableStore = Union[str, MutableMapping[str, bytes]]
+
+
+def _infer_chunks(dataset: xarray.Dataset) -> Mapping[str, int]:
+  """Infer chunks for an xarray.Dataset loaded from Zarr."""
+  # The original Zarr array chunks (as tuples) are stored in the "encoding"
+  # dictionary on each DataArray object.
+
+  chunks_sets = collections.defaultdict(set)
+  for name, variable in dataset.items():
+    # exclude variables that are indexed, which are loaded into memory already
+    if name not in dataset.indexes:
+      for dim, size in zip(variable.dims, variable.encoding['chunks']):
+        chunks_sets[dim].add(size)
+
+  chunks = {}
+  for dim, sizes in chunks_sets.items():
+    if len(sizes) > 1:
+      raise ValueError(
+          f'inconsistent chunk sizes on Zarr dataset for dimension {dim!r}: '
+          f'{sizes}'
+      )
+    (chunks[dim],) = sizes
+  return chunks
+
+
+def open_zarr(
+    store: ReadableStore, **kwargs: Any
+) -> Tuple[xarray.Dataset, Mapping[str, int]]:
+  """Returns a lazily indexable xarray.Dataset and chunks from a Zarr store.
+
+  Only Zarr stores with the consistent chunking between non-indexed variables
+  (i.e., those for which the ``Dataset.chunks`` property is valid) can be
+  opened.
+
+  Args:
+    store: Xarray compatible Zarr store to open.
+    **kwargs: passed on to xarray.open_zarr. The "chunks" keyword argument is
+      not supported.
+
+  Returns:
+    (dataset, chunks) pair, consisting of a Dataset with the contents of the
+    Zarr store and a mapping from dimensions to integer chunk sizes.
+  """
+  if 'chunks' in kwargs:
+    raise TypeError(
+        'xarray_beam.open_zarr does not support the `chunks` argument'
+    )
+  dataset = xarray.open_zarr(store, **kwargs, chunks=None)
+  chunks = _infer_chunks(dataset)
+  return dataset, chunks
+
+
+def make_template(dataset: xarray.Dataset) -> xarray.Dataset:
+  """Make a lazy Dask xarray.Dataset of all zeros."""
+  return xarray.zeros_like(dataset.chunk(-1))
 
 
 class _DiscoverTemplate(beam.PTransform):
   """Discover the Zarr template from (Key, xarray.Dataset) pairs."""
 
   def _make_template_chunk(self, key, chunk):
-    # Make a lazy Dask xarray.Dataset of all zeros shaped like this chunk.
     # The current rule is that everything that *can* be chunked with Dask
     # *should* be chunked, but conceivably this should be customizable, e.g.,
     # for handling 2D latitude/longitude arrays. It's usually safe to rewrite
     # overlapping arrays multiple times in different chunks (Zarr writes are
     # typically atomic), but this may be wasteful.
-    zeros = xarray.zeros_like(chunk.chunk(-1))
-    return key, zeros
+    return key, make_template(chunk)
 
   def _consolidate(self, inputs):
     # don't bother with compatibility checks; we won't be computing the values
     # here anyways
     kwargs = {'compat': 'override'}
     _, template = rechunk.consolidate_fully(
-        inputs, combine_kwargs=kwargs, merge_kwargs=kwargs,
+        inputs, combine_kwargs=kwargs, merge_kwargs=kwargs
     )
     return template
 
@@ -63,7 +120,9 @@ def _verify_template_is_lazy(template: xarray.Dataset):
     # We require at least one chunked variable with Dask. Otherwise, there would
     # be no data to write as part of the Beam pipeline.
     raise ValueError(
-        f'template does not have any variables chunked with Dask:\n{template}'
+        'template does not have any variables chunked with Dask. Convert any '
+        'variables that will be written in the pipeline into lazy dask arrays, '
+        f'e.g., with xarray_beam.make_template():\n{template}'
     )
 
 
@@ -99,12 +158,10 @@ def _setup_zarr(template, store, zarr_chunks):
     if 'chunks' in var.encoding:
       del var.encoding['chunks']
   logging.info(f'writing Zarr metadata for template:\n{template}')
-  template2.to_zarr(
-      store, compute=False, consolidated=True, mode='w',
-  )
+  template2.to_zarr(store, compute=False, consolidated=True, mode='w')
 
 
-def _validate_zarr_chunk(key, chunk, template):
+def _validate_zarr_chunk(key, chunk, template, zarr_chunks):
   """Check a chunk for consistency against the given template."""
   unexpected_indexes = [k for k in chunk.indexes if k not in template.indexes]
   if unexpected_indexes:
@@ -112,6 +169,7 @@ def _validate_zarr_chunk(key, chunk, template):
         'unexpected new indexes found in chunk but not template: '
         f'{unexpected_indexes}'
     )
+
   region = core.offsets_to_slices(key.offsets, chunk.sizes)
   for dim, full_index in template.indexes.items():
     if dim in chunk.indexes:
@@ -119,9 +177,32 @@ def _validate_zarr_chunk(key, chunk, template):
       actual_index = chunk.indexes[dim]
       if not expected_index.equals(actual_index):
         raise ValueError(
-            f'template and chunk indexes do not match for dim {dim}:\n'
+            f'template and chunk indexes do not match for dim {dim!r}:\n'
             f'{expected_index}\nvs.\n{actual_index}'
         )
+
+  if zarr_chunks is None:
+    zarr_chunks = {k: v[0] for k, v in template.chunks.items()}
+  for dim, offset in key.offsets.items():
+    if dim not in zarr_chunks:
+      continue
+    if offset % zarr_chunks[dim]:
+      raise ValueError(
+          f'chunk offset {offset} along dimension {dim!r} is not a multiple of '
+          f'zarr chunks {zarr_chunks}'
+      )
+    if (
+        chunk.sizes[dim] % zarr_chunks[dim]
+        and offset + chunk.sizes[dim] != template.sizes[dim]
+    ):
+      raise ValueError(
+          f'chunk is smaller than zarr chunks {zarr_chunks} along at least one '
+          'dimension, which can lead to a race condition that results in '
+          'incomplete writes. Use ConsolidateChunks() or Rechunk() to ensure '
+          'appropriate chunk sizes before feeding data into ChunksToZarr(). '
+          f'\nkey={key}\nchunk={chunk}'
+      )
+
   # TODO(shoyer): consider verifying "already_written" variables for
   # consistency, maybe with an opt-in flag?
   # Note that variable names, shapes & dtypes are verified in xarray's to_zarr()
@@ -139,8 +220,9 @@ def _write_chunk_to_zarr(key, chunk, store, template):
   ]
   writable_chunk = chunk.drop_vars(already_written)
   try:
-    future = writable_chunk.chunk().to_zarr(store, region=region, compute=False,
-                                            consolidated=True)
+    future = writable_chunk.chunk().to_zarr(
+        store, region=region, compute=False, consolidated=True
+    )
     future.compute(num_workers=len(writable_chunk))
   except Exception as e:
     raise RuntimeError(
@@ -153,7 +235,7 @@ class ChunksToZarr(beam.PTransform):
 
   def __init__(
       self,
-      store: Union[str, MutableMapping[str, bytes]],
+      store: WritableStore,
       template: Union[xarray.Dataset, beam.pvalue.AsSingleton, None] = None,
       zarr_chunks: Optional[Mapping[str, int]] = None,
       num_threads: Optional[int] = None,
@@ -162,11 +244,12 @@ class ChunksToZarr(beam.PTransform):
 
     Args:
       store: a string corresponding to a Zarr path or an existing Zarr store.
-      template: an argument providing an xarray.Dataset already chunked using
-        Dask that matches the structure of xarray.Dataset "chunks" that will be
-        fed into this PTransform. One or more variables are expected to be
-        "chunked" with Dask, and will only have their metadata written to Zarr
-        without array values. Three types of inputs are supported:
+      template: an argument providing a lazy xarray.Dataset already chunked
+        using Dask (e.g., as created by `xarray_beam.make_template`) that
+        matches the structure of the virtual combined dataset corresponding to
+        the chunks fed into this PTransform. One or more variables are expected
+        to be "chunked" with Dask, and will only have their metadata written to
+        Zarr without array values. Three types of inputs are supported:
 
         1. If `template` is an xarray.Dataset, the Zarr store is setup eagerly.
         2. If `template` is a beam.pvalue.AsSingleton object representing the
@@ -207,7 +290,7 @@ class ChunksToZarr(beam.PTransform):
     # must have defaults for MapTuple". Beam should probably be happy with a
     # keyword-only argument, too, but it doesn't like that yet.
     assert template is not None
-    _validate_zarr_chunk(key, chunk, template)
+    _validate_zarr_chunk(key, chunk, template, self.zarr_chunks)
     return key, chunk
 
   def _write_chunk_to_zarr(self, key, chunk, template=None):
@@ -217,15 +300,14 @@ class ChunksToZarr(beam.PTransform):
   def expand(self, pcoll):
     if isinstance(self.template, xarray.Dataset):
       template = self.template
-      setup_result = None
+      setup_result = None  # already setup in __init__
     else:
       if isinstance(self.template, beam.pvalue.AsSingleton):
         template = self.template
       else:
         assert self.template is None
         template = beam.pvalue.AsSingleton(
-            pcoll
-            | 'DiscoverTemplate' >> _DiscoverTemplate()
+            pcoll | 'DiscoverTemplate' >> _DiscoverTemplate()
         )
       setup_result = beam.pvalue.AsSingleton(
           template.pvalue
@@ -234,10 +316,10 @@ class ChunksToZarr(beam.PTransform):
     return (
         pcoll
         | 'WaitForSetup' >> beam.Map(lambda x, _: x, setup_result)
-        | 'ValidateChunks' >> beam.MapTuple(
-            self._validate_zarr_chunk, template=template,
-        )
-        | 'WriteChunks' >> threadmap.ThreadMapTuple(
+        | 'ValidateChunks'
+        >> beam.MapTuple(self._validate_zarr_chunk, template=template)
+        | 'WriteChunks'
+        >> threadmap.ThreadMapTuple(
             self._write_chunk_to_zarr,
             template=template,
             num_threads=self.num_threads,
@@ -248,8 +330,9 @@ class ChunksToZarr(beam.PTransform):
 @dataclasses.dataclass
 class DatasetToZarr(beam.PTransform):
   """Write an entire xarray.Dataset to a Zarr store."""
+
   dataset: xarray.Dataset
-  store: Union[str, MutableMapping[str, bytes]]
+  store: WritableStore
   zarr_chunks: Optional[Mapping[str, int]] = None
 
   def expand(self, pcoll):
@@ -260,6 +343,6 @@ class DatasetToZarr(beam.PTransform):
         pcoll
         | core.DatasetToChunks(source_dataset)
         | ChunksToZarr(
-            self.store, template=self.dataset, zarr_chunks=self.zarr_chunks,
+            self.store, template=self.dataset, zarr_chunks=self.zarr_chunks
         )
     )
